@@ -94,7 +94,7 @@ int md_get_version(int fd)
 	return  (vers.major*10000) + (vers.minor*100) + vers.patchlevel;
     if (errno == EACCES)
 	    return -1;
-    if (MAJOR(stb.st_rdev) == MD_MAJOR)
+    if (major(stb.st_rdev) == MD_MAJOR)
 	return (3600);
     return -1;
 }
@@ -278,6 +278,7 @@ int store_super(int fd, mdp_super_t *super)
 	if (write(fd, super, sizeof(*super)) != sizeof(*super))
 		return 4;
 
+	fsync(fd);
 	return 0;
 }
     
@@ -395,28 +396,37 @@ int map_name(mapping_t *map, char *name)
 }
 
 
-int is_standard(char *dev)
+int is_standard(char *dev, int *nump)
 {
 	/* tests if dev is a "standard" md dev name.
 	 * i.e if the last component is "/dNN" or "/mdNN",
 	 * where NN is a string of digits 
 	 */
-	dev = strrchr(dev, '/');
-	if (!dev)
+	char *d = strrchr(dev, '/');
+	int type=0;
+	int num;
+	if (!d)
 		return 0;
-	if (strncmp(dev, "/d",2)==0)
-		dev += 2;
-	else if (strncmp(dev, "/md", 3)==0)
-		dev += 3;
+	if (strncmp(d, "/d",2)==0)
+		d += 2, type=1; /* /dev/md/dN{pM} */
+	else if (strncmp(d, "/md_d", 5)==0)
+		d += 5, type=1; /* /dev/md_dNpM */
+	else if (strncmp(d, "/md", 3)==0)
+		d += 3, type=-1; /* /dev/mdN */
+	else if (d-dev > 3 && strncmp(d-2, "md/", 3)==0)
+		type = -1; /* /dev/md/N */
 	else
 		return 0;
-	if (!*dev)
+	if (!*d)
 		return 0;
-	while (isdigit(*dev))
-		dev++;
-	if (*dev)
+	num = atoi(d);
+	while (isdigit(*d))
+		d++;
+	if (*d)
 		return 0;
-	return 1;
+	if (nump) *nump = num;
+
+	return type;
 }
 
 
@@ -456,8 +466,8 @@ int add_dev(const char *name, const struct stat *stb, int flag)
 	char *n = strdup(name);
 	struct devmap *dm = malloc(sizeof(*dm));
 	if (dm) {
-	    dm->major = MAJOR(stb->st_rdev);
-	    dm->minor = MINOR(stb->st_rdev);
+	    dm->major = major(stb->st_rdev);
+	    dm->minor = minor(stb->st_rdev);
 	    dm->name = n;
 	    dm->next = devlist;
 	    devlist = dm;
@@ -468,7 +478,7 @@ int add_dev(const char *name, const struct stat *stb, int flag)
 
 /*
  * Find a block device with the right major/minor number.
- * Avoid /dev/mdNN and /dev/md/dNN is possible
+ * Avoid /dev/mdNN and /dev/md/dNN if possible
  */
 char *map_dev(int major, int minor)
 {
@@ -486,7 +496,7 @@ char *map_dev(int major, int minor)
 	for (p=devlist; p; p=p->next)
 		if (p->major == major &&
 		    p->minor == minor) {
-			if (is_standard(p->name))
+			if (is_standard(p->name, NULL))
 				std = p->name;
 			else
 				return p->name;
@@ -600,14 +610,14 @@ char *get_md_name(int dev)
 	if (dev < 0) {
 		int mdp =  get_mdp_major();
 		if (mdp < 0) return NULL;
-		rdev = MKDEV(mdp, (-1-dev)<<6);
+		rdev = makedev(mdp, (-1-dev)<<6);
 		sprintf(devname, "/dev/md/d%d", -1-dev);
 		if (stat(devname, &stb) == 0
 		    && (S_IFMT&stb.st_mode) == S_IFBLK
 		    && (stb.st_rdev == rdev))
 			return devname;
 	} else {
-		rdev = MKDEV(MD_MAJOR, dev);
+		rdev = makedev(MD_MAJOR, dev);
 		sprintf(devname, "/dev/md%d", dev);
 		if (stat(devname, &stb) == 0
 		    && (S_IFMT&stb.st_mode) == S_IFBLK
@@ -620,7 +630,7 @@ char *get_md_name(int dev)
 		    && (stb.st_rdev == rdev))
 			return devname;
 	}
-	dn = map_dev(MAJOR(rdev), MINOR(rdev));
+	dn = map_dev(major(rdev), minor(rdev));
 	if (dn)
 		return dn;
 	sprintf(devname, "/dev/.tmp.md%d", dev);
